@@ -179,189 +179,217 @@ target field is set to the symbol `none'.")
 (defconst parsebib--key-regexp "[^\"@\\#%',={} \t\n\f]+" "Regexp describing a licit key.")
 (defconst parsebib--entry-start "^[ \t]*@" "Regexp describing the start of an entry.")
 
-(defun parsebib--build-TeX-accent-command-regexp (command accent)
-  "Build a regexp-replacement pair for LaTeX diacritics.
-
-COMMAND is the name of a TeX or LaTeX command (without
-backslash), ACCENT is the character (usually a Unicode combining
-character) that COMMAND generates.  Both COMMAND and ACCENT must
-be strings.
-
-The return value is a cons cell that can be included in
-`parsebib-TeX-markup-replace-alist' directly.
-
-The car of this cons cell is a regexp matching the TeX or LaTeX
-COMMAND, capturing exactly one obligatory argument.  The
-cdr is a replacement string, the concatenation of \"\\1\" and
-ACCENT.
-
-Specifically, the car regexp matches a string composed of a
-backslash, followed by COMMAND and a single letter (i.e.
-matching [[:alpha:]]).  The regexp matches if the letter is in
-curly braces (\"\\d{a}\") or if it is separated from COMMAND by
-white space (\"\\d a\".  If COMMAND is a non-letter character,
-the regexp also matches if the letter follows COMMAND
-immediately, without white space or curly braces (\"\\'a\").  In
-all variants, the letter is captured with group number 1."
-  (cons
-   (rx-to-string
-    `(: "\\" ,command
-        (or (: (* blank) "{" (group-n 1 letter) "}")
-            (: (,(if (string-match "[a-zA-Z]" command) '+ '*) blank)
-               (group-n 1 letter))))
-    t)
-   (rx-to-string `(: (backref 1) ,accent) t)))
-
-(defun parsebib--build-TeX-command-regexp (command replacement)
-  "Build a regexp-replacement pair for a LaTeX command.
-
-COMMAND is the name of a TeX or LaTeX command (without
-backslash).  Both COMMAND and REPLACEMENT must be strings.
-
-The return value is a cons cell: its car is a regexp matching
-COMMAND, its cdr is REPLACEMENT.  This cons cell can be included
-in `parsebib-TeX-markup-replace-alist' directly.
-
-Specifically, the regexp matches a string composed of a backslash
-followed by COMMAND and terminated by a pair of curly
-braces (`\\COMMAND{}'), a word ending or a space.  Such a
-trailing space will be included in the overall match."
-  (cons
-   (rx-to-string
-    `(: "\\" ,(if (listp command) `(or ,@command) command)
-        ;; If a command is terminated by a space, LaTeX includes that
-        ;; space in the command itself, so it is not printed (like the
-        ;; behaviour for a following {}) Accordingly, if there is one,
-        ;; include that space in the replaced string by matching on it
-        ;; first.
-        (or (+ blank) word-end "{}"))
-    t)
-   replacement))
-
 (defun parsebib--convert-tex-italics (str)
-  "Return first sub-expression match in STR, in italics."
-  (propertize (match-string 1 str) 'face 'italic))
+  "Return STR with face property `italic'."
+  (propertize str 'face 'italic))
 
 (defun parsebib--convert-tex-bold (str)
-  "Return first sub-expression match in STR, in bold."
-  (propertize (match-string 1 str) 'face 'bold))
+  "Return STR with face property `bold'."
+  (propertize str 'face 'bold))
 
 (defun parsebib--convert-tex-small-caps (str)
-  "Return first sub-expression match in STR, capitalised."
-  (upcase (match-string 1 str)))
+  "Return STR capitalised."
+  (upcase str))
 
-(defvar parsebib-TeX-markup-replace-alist
-  `(;; Commands defined to work in both math and text mode.  (Dashes are
-    ;; separate because they are not backslash-escaped, unlike everything else.)
-    ("---\\|\\\\textemdash\\(?: +\\|{}\\|\\>\\)" . "\N{EM DASH}")
-    ("--\\|\\\\textendash\\(?: +\\|{}\\|\\>\\)"  . "\N{EN DASH}")
-    ,@(mapcar
-       (apply-partially 'apply 'parsebib--build-TeX-command-regexp)
-       '((("ddag" "textdaggerdbl")        "\N{DOUBLE DAGGER}")
-         (("dag" "textdagger")            "\N{DAGGER}")
-         ("textpertenthousand"            "\N{PER TEN THOUSAND SIGN}")
-         ("textperthousand"               "\N{PER MILLE SIGN}")
-         ("textquestiondown"              "\N{INVERTED QUESTION MARK}")
-         ("P"                             "\N{PILCROW SIGN}")
-         ("textdollar"                    "$")
-         ("S"                             "\N{SECTION SIGN}")
-         (("ldots" "dots" "textellipsis") "\N{HORIZONTAL ELLIPSIS}")))
+(defvar parsebib-TeX-command-replacement-alist
+  '(("ddag" . "\N{DOUBLE DAGGER}")
+    ("textdaggerdbl" . "\N{DOUBLE DAGGER}")
+    ("dag" . "\N{DAGGER}")
+    ("textdagger" . "\N{DAGGER}")
+    ("textpertenthousand" . "\N{PER TEN THOUSAND SIGN}")
+    ("textperthousand" . "\N{PER MILLE SIGN}")
+    ("textquestiondown" . "\N{INVERTED QUESTION MARK}")
+    ("P" . "\N{PILCROW SIGN}")
+    ("textdollar" . "$")
+    ("S" . "\N{SECTION SIGN}")
+    ("ldots" . "\N{HORIZONTAL ELLIPSIS}")
+    ("dots" . "\N{HORIZONTAL ELLIPSIS}")
+    ("textellipsis" . "\N{HORIZONTAL ELLIPSIS}")
+    ("textemdash" . "\N{EM DASH}")
+    ("textendash" . "\N{EN DASH}")
 
-    ;; Text-mode Accents
-    ,@(mapcar
-       (apply-partially 'apply 'parsebib--build-TeX-accent-command-regexp)
-       '(("\"" "\N{COMBINING DIAERESIS}")
-         ("'"  "\N{COMBINING ACUTE ACCENT}")
-         ("."  "\N{COMBINING DOT ABOVE}")
-         ("="  "\N{COMBINING MACRON}")
-         ("^"  "\N{COMBINING CIRCUMFLEX ACCENT}")
-         ("`"  "\N{COMBINING GRAVE ACCENT}")
-         ("b"  "\N{COMBINING MACRON BELOW}")
-         ("c"  "\N{COMBINING CEDILLA}")
-         ("d"  "\N{COMBINING DOT BELOW}")
-         ("H"  "\N{COMBINING DOUBLE ACUTE ACCENT}")
-         ("k"  "\N{COMBINING OGONEK}")
-         ("U"  "\N{COMBINING DOUBLE VERTICAL LINE ABOVE}")
-         ("u"  "\N{COMBINING BREVE}")
-         ("v"  "\N{COMBINING CARON}")
-         ("~"  "\N{COMBINING TILDE}")
-         ("|"  "\N{COMBINING COMMA ABOVE}")
-         ("f"  "\N{COMBINING INVERTED BREVE}")
-         ("G"  "\N{COMBINING DOUBLE GRAVE ACCENT}")
-         ("h"  "\N{COMBINING HOOK ABOVE}")
-         ("C"  "\N{COMBINING DOUBLE GRAVE ACCENT}")
-         ("r"  "\N{COMBINING RING ABOVE}")))
+    ;; Non-ASCII Letters (Excluding Accented Letters)
+    ("AA" . "\N{LATIN CAPITAL LETTER A WITH RING ABOVE}")
+    ("AE" . "\N{LATIN CAPITAL LETTER AE}")
+    ("DH" . "\N{LATIN CAPITAL LETTER ETH}")
+    ("DJ" . "\N{LATIN CAPITAL LETTER ETH}")
+    ("L"  . "\N{LATIN CAPITAL LETTER L WITH STROKE}")
+    ("SS" . "\N{LATIN CAPITAL LETTER SHARP S}")
+    ("NG" . "\N{LATIN CAPITAL LETTER ENG}")
+    ("OE" . "\N{LATIN CAPITAL LIGATURE OE}")
+    ("O"  . "\N{LATIN CAPITAL LETTER O WITH STROKE}")
+    ("TH" . "\N{LATIN CAPITAL LETTER THORN}")
 
-    ;; LaTeX2 Escapable "Special" Characters
-    ("\\\\%" . "%") ("\\\\&" . "&") ("\\\\#" . "#") ("\\\\\\$" . "$")
+    ("aa" . "\N{LATIN SMALL LETTER A WITH RING ABOVE}")
+    ("ae" . "\N{LATIN SMALL LETTER AE}")
+    ("dh" . "\N{LATIN SMALL LETTER ETH}")
+    ("dj" . "\N{LATIN SMALL LETTER ETH}")
+    ("l"  . "\N{LATIN SMALL LETTER L WITH STROKE}")
+    ("ss" . "\N{LATIN SMALL LETTER SHARP S}")
+    ("ng" . "\N{LATIN SMALL LETTER ENG}")
+    ("oe" . "\N{LATIN SMALL LIGATURE OE}")
+    ("o"  . "\N{LATIN SMALL LETTER O WITH STROKE}")
+    ("th" . "\N{LATIN SMALL LETTER THORN}")
 
+    ("ij" . "ij")
+    ("i" . "\N{LATIN SMALL LETTER DOTLESS I}")
+    ("j" . "\N{LATIN SMALL LETTER DOTLESS J}")
+    ;; Formatting Commands
+    ("textit" . parsebib--convert-tex-italics)
+    ("emph"   . parsebib--convert-tex-italics)
+    ("textbf" . parsebib--convert-tex-bold)
+    ("textsc" . parsebib--convert-tex-small-caps))
+  "An alist of <command>-<replacement> pairs for LaTeX commands.
+<command> is the name of a TeX or LaTeX command (without
+backslash), <replacement> is the string with which it is
+replaced.
+
+<replacement> can also be a function of one argument.  In this
+case, <command> must take at least one obligatory argument, which
+is passed as the first argument of the replacement function.  The
+return value of this function is used as the replacement string
+for <command>.
+
+See `parsebib-TeX-markup-replacement-alist' and the function
+`parsebib-clean-TeX-markup' to see how this variable is used.")
+
+(defvar parsebib-TeX-accent-replacement-alist
+  '(("\"" . "\N{COMBINING DIAERESIS}")
+    ("'" . "\N{COMBINING ACUTE ACCENT}")
+    ("." . "\N{COMBINING DOT ABOVE}")
+    ("=" . "\N{COMBINING MACRON}")
+    ("^" . "\N{COMBINING CIRCUMFLEX ACCENT}")
+    ("`" . "\N{COMBINING GRAVE ACCENT}")
+    ("b" . "\N{COMBINING MACRON BELOW}")
+    ("c" . "\N{COMBINING CEDILLA}")
+    ("d" . "\N{COMBINING DOT BELOW}")
+    ("H" . "\N{COMBINING DOUBLE ACUTE ACCENT}")
+    ("k" . "\N{COMBINING OGONEK}")
+    ("U" . "\N{COMBINING DOUBLE VERTICAL LINE ABOVE}")
+    ("u" . "\N{COMBINING BREVE}")
+    ("v" . "\N{COMBINING CARON}")
+    ("~" . "\N{COMBINING TILDE}")
+    ("|" . "\N{COMBINING COMMA ABOVE}")
+    ("f" . "\N{COMBINING INVERTED BREVE}")
+    ("G" . "\N{COMBINING DOUBLE GRAVE ACCENT}")
+    ("h" . "\N{COMBINING HOOK ABOVE}")
+    ("C" . "\N{COMBINING DOUBLE GRAVE ACCENT}")
+    ("r" . "\N{COMBINING RING ABOVE}") )
+"Alist of <command>-<accent> pairs for LaTeX diacritics.
+<command> is the name of a TeX or LaTeX command (without
+backslash), <accent> is the Unicode combining character for the
+diacritic that <command> generates.  Both <command> and <accent>
+must be strings.
+
+The replacement string for <command> is composed of its
+obligatory argument (usually a single character) and the
+combining diacritic.
+
+See `parsebib-TeX-markup-replacement-alist' and the function
+`parsebib-clean-TeX-markup' to see how this variable is used.")
+
+(defvar parsebib-TeX-literal-replacement-alist
+  ;; LaTeX2 Escapable "Special" Characters
+  `(("\\%" . "%") ("\\&" . "&") ("\\#" . "#") ("\\$" . "$")
     ;; Quotes
     ("``" . "\N{LEFT DOUBLE QUOTATION MARK}")
     ("`"  . "\N{LEFT SINGLE QUOTATION MARK}")
     ("''" . "\N{RIGHT DOUBLE QUOTATION MARK}")
     ("'"  . "\N{RIGHT SINGLE QUOTATION MARK}")
-
-    ;; Formatting Commands
-    ("\\\\textit{\\(.*?\\)}" . parsebib--convert-tex-italics)
-    ("\\\\emph{\\(.*?\\)}"   . parsebib--convert-tex-italics)
-    ("\\\\textbf{\\(.*?\\)}" . parsebib--convert-tex-bold)
-    ("\\\\textsc{\\(.*?\\)}" . parsebib--convert-tex-small-caps)
-
-    ;; Non-ASCII Letters (Excluding Accented Letters)
-    ,@(mapcar
-       (apply-partially 'apply 'parsebib--build-TeX-command-regexp)
-       '(("AA" "\N{LATIN CAPITAL LETTER A WITH RING ABOVE}")
-         ("AE" "\N{LATIN CAPITAL LETTER AE}")
-         ("DH" "\N{LATIN CAPITAL LETTER ETH}")
-         ("DJ" "\N{LATIN CAPITAL LETTER ETH}")
-         ("L"  "\N{LATIN CAPITAL LETTER L WITH STROKE}")
-         ("SS" "\N{LATIN CAPITAL LETTER SHARP S}")
-         ("NG" "\N{LATIN CAPITAL LETTER ENG}")
-         ("OE" "\N{LATIN CAPITAL LIGATURE OE}")
-         ("O"  "\N{LATIN CAPITAL LETTER O WITH STROKE}")
-         ("TH" "\N{LATIN CAPITAL LETTER THORN}")
-
-         ("aa" "\N{LATIN SMALL LETTER A WITH RING ABOVE}")
-         ("ae" "\N{LATIN SMALL LETTER AE}")
-         ("dh" "\N{LATIN SMALL LETTER ETH}")
-         ("dj" "\N{LATIN SMALL LETTER ETH}")
-         ("l"  "\N{LATIN SMALL LETTER L WITH STROKE}")
-         ("ss" "\N{LATIN SMALL LETTER SHARP S}")
-         ("ng" "\N{LATIN SMALL LETTER ENG}")
-         ("oe" "\N{LATIN SMALL LIGATURE OE}")
-         ("o"  "\N{LATIN SMALL LETTER O WITH STROKE}")
-         ("th" "\N{LATIN SMALL LETTER THORN}")
-
-         ("ij" "ij")
-         ("i"  "\N{LATIN SMALL LETTER DOTLESS I}")
-         ("j"  "\N{LATIN SMALL LETTER DOTLESS J}")))
-
-    ;; Commands with obligatory non-empty argument
-    ("\\\\[a-zA-Z*]+\\(?:\\[.*\\]\\)?{\\(.+?\\)}" . "\\1")
-
-    ;; Commands without arguments, optionally terminated by empty braces
-    ("\\(\\\\[a-zA-Z*]+\\)\\(?:\\[.*\\]\\)?\\(?:{}\\)?" . "\\1")
-
-    ;; Collapse white space
-    ("[[:blank:]]+" . " ")
-
+    ;; Dashes
+    ("---" . "\N{EM DASH}")
+    ("--" . "\N{EN DASH}")
     ;; Remove all remaining {braces}
     ("{" . "") ("}" . ""))
-  "Alist of strings and replacements for TeX markup.
+  "Alist of <literal>-<replacement> pairs.  Both are strings.
+This variable contains characters that are special in LaTeX and
+single-character, non-ASCII LaTeX commands.
+
+Note that adding pairs to this variable has no effect unless
+`parsebib-TeX-markup-replacement-alist' is adjusted accordingly.
+For example, after adding a <literal>-<replacement> pair, the
+following code will ensure that <literal> gets replaced with
+<replacement>.
+
+  (cl-callf (lambda (regex) (rx (or <literal> (regexp regex))))
+     (alist-get (quote parsebib--replace-literal)
+                parsebib-TeX-markup-replacement-alist))
+
+See `parsebib-TeX-markup-replacement-alist' and the function
+`parsebib-clean-TeX-markup' to see how this variable is used.")
+
+(defvar parsebib-TeX-markup-replacement-alist
+  `((parsebib--replace-command-or-accent
+     ;; This regexp matches any latex command i.e. anything that
+     ;; starts with a backslash. The name of the command which
+     ;; is either a string of alphabetic characters or a single
+     ;; non-alphabetic character is captured by group 1. The command
+     ;; can have a mandatory argument enclosed by braces which is
+     ;; captured by group 2. If the command has no arguments in
+     ;; brackets or braces, the first non-white space letter after
+     ;; the command is captured in group 3. This is to be able to deal
+     ;; with accents.
+     ;; Note that the capturing of arguments in braces is imperfect,
+     ;; because doing it properly requires sexp parsing. It will fail
+     ;; for cases like \command{\anothercommand{an arg}some text}.
+     . ,(rx "\\" (group-n 1 (or (1+ letter) nonl))
+          (: (* blank) (opt (or (: (* (: "[" (* (not "]")) "]"))
+                                 "{" (group-n 2 (0+ (not "}"))) (opt "}"))
+                                (group-n 3 letter))))))
+    (parsebib--replace-literal
+     . ,(rx-to-string `(or ,@(mapcar #'car parsebib-TeX-literal-replacement-alist)
+                           (1+ blank)))))
+  "Alist of replacements and strings for TeX markup.
 This is used in `parsebib-clean-TeX-markup' to make TeX markup more
-suitable for display.  Each item in the list consists of a regexp
-and its replacement.  The replacement can be a string (which will
+suitable for display.  Each item in the list consists of a replacement
+and a regexp.  The replacement can be a string (which will
 simply replace the match) or a function (the match will be
 replaced by the result of calling the function on the match
 string).  Earlier elements are evaluated before later ones, so if
 one string is a subpattern of another, the second must appear
-later (e.g. \"''\" is before \"'\").")
+later (e.g. \"''\" is before \"'\").
+
+For the common cases of replacing a LaTeX command or a literal
+it is faster to use `parsebib-TeX-command-replacement-alist'
+and `parsebib-TeX-literal-replacement-alist' respectively.")
 
 (defvar parsebib-clean-TeX-markup-excluded-fields '("file"
                                                     "url"
                                                     "doi")
   "List of fields that should not be passed to `parsebib-clean-TeX-markup'.")
 
+(defun parsebib--replace-command-or-accent (string)
+  "Return the replacement text for the command or accent matched by STRING."
+  (let* ((cmd (match-string 1 string))
+         ;; bar is the argument in braces.
+         (bar (match-string 2 string))
+         ;; If there is no argument in braces, consider the letter after
+         ;; the command as the argument. Clean this argument.
+         (arg (parsebib-clean-TeX-markup (or (if bar bar (match-string 3 string)) "")))
+         ;; Check if the cmd is an accent that needs to be replaced
+         ;; and get its replacement.
+         (acc (alist-get cmd parsebib-TeX-accent-replacement-alist nil nil #'equal))
+         ;; If it is not an accent, check if it is a command that needs to be replaced
+         ;; and get the replacement.
+         (rep (or acc (alist-get cmd parsebib-TeX-command-replacement-alist nil nil #'equal))))
+    (cond
+     ;; If replacement is a function call it with the argument.
+     ((functionp rep) (funcall rep arg))
+     ;; Otherwise combine the replacement with the argument. The order of combination
+     ;; depends on whether the command is an accent or not.
+     (rep (if acc (concat arg rep) (concat rep arg)))
+     ;; Now we handle the fallback cases. If there is a braced argument but no
+     ;; replacement for the command was found, consider the replacement to be
+     ;; empty.
+     ((and bar (not (equal "" bar))) bar)
+     ;; Otherwise clean any optional arguments by discarding them.
+     (t (replace-regexp-in-string (rx "[" (* (not "]")) "]") "" string t t)))))
+
+(defun parsebib--replace-literal (string)
+  "Look up the replacement text for literal STRING."
+  (or (alist-get string parsebib-TeX-literal-replacement-alist nil nil #'equal)
+      " "))
+
 (defun parsebib-clean-TeX-markup (string)
   "Return STRING without TeX markup.
 Any substring matching the car of a cell in
@@ -370,12 +398,11 @@ corresponding cdr (if the cdr is a string), or with the result of
 calling the cdr on the match (if it is a function).  This is done
 with `replace-regexp-in-string', which see for details."
   (let ((case-fold-search nil))
-    (save-match-data
-      (cl-loop for (pattern . replacement) in parsebib-TeX-markup-replace-alist
-               do (setq string (replace-regexp-in-string
-                                pattern replacement string
-                                nil (functionp replacement)))
-               finally return string))))
+    (cl-loop for (replacement . pattern) in parsebib-TeX-markup-replacement-alist
+             do (setq string (replace-regexp-in-string
+                              pattern replacement string
+                              t t))
+             finally return string)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Matching and parsing stuff ;;
